# -*- coding: utf-8 -*-
"""Asynchronous Python client for the AdGuard Home API."""

from .exceptions import AdGuardHomeError


class AdGuardHomeSafeBrowsing:
    """Controls AdGuard Home browsing security."""

    def __init__(self, adguard):
        """Initialize object."""
        self._adguard = adguard

    async def enabled(self) -> bool:
        """Return if AdGuard Home browsing security is enabled or not."""
        response = await self._adguard._request("safebrowsing/status")
        return response["enabled"]

    async def enable(self) -> bool:
        """Enable AdGuard Home browsing security."""
        response = await self._adguard._request("safebrowsing/enable", method="POST")
        if response.rstrip() != "OK":
            raise AdGuardHomeError(
                "Enabling AdGuard Home safe browsing failed", {"response": response}
            )
        return True

    async def disable(self) -> bool:
        """Disable AdGuard Home browsing security."""
        response = await self._adguard._request("safebrowsing/disable", method="POST")
        if response.rstrip() != "OK":
            raise AdGuardHomeError(
                "Disabling AdGuard Home safe browsing failed", {"response": response}
            )
        return True



# -*- coding: utf-8 -*-
"""Asynchronous Python client for the AdGuard Home API."""

from .exceptions import AdGuardHomeError


class AdGuardHomeFiltering:
    """Controls AdGuard Home filtering. Blocks domains."""

    def __init__(self, adguard):
        """Initialize object."""
        self._adguard = adguard

    async def enabled(self) -> bool:
        """Return if AdGuard Home filtering is enabled or not."""
        response = await self._adguard._request("filtering/status")
        return response["enabled"]

    async def enable(self) -> bool:
        """Enable AdGuard Home filtering."""
        response = await self._adguard._request("filtering/enable", method="POST")
        if response.rstrip() != "OK":
            raise AdGuardHomeError(
                "Enabling AdGuard Home filtering failed", {"response": response}
            )
        return True

    async def disable(self) -> bool:
        """Disable AdGuard Home filtering."""
        response = await self._adguard._request("filtering/disable", method="POST")
        if response.rstrip() != "OK":
            raise AdGuardHomeError(
                "Disabling AdGuard Home filtering failed", {"response": response}
            )
        return True

    async def rules_count(self) -> int:
        """Return the number of rules loaded."""
        response = await self._adguard._request("filtering/status")
        count = 0
        for filt in response["filters"]:
            count += filt["rulesCount"]
        return count

    async def add_url(self, name: str, url: str) -> bool:
        """Add a new filter subscription to AdGuard Home."""
        response = await self._adguard._request(
            "filtering/add_url", method="POST", json_data={"name": name, "url": url}
        )
        if not response.startswith("OK"):
            raise AdGuardHomeError(
                "Failed adding URL to AdGuard Home filter", {"response": response}
            )
        return True

    async def remove_url(self, url: str) -> bool:
        """Remove a new filter subscription from AdGuard Home."""
        response = await self._adguard._request(
            "filtering/remove_url", method="POST", data="url={}".format(url)
        )
        if response.rstrip() != "OK":
            raise AdGuardHomeError(
                "Failed removing URL from AdGuard Home filter", {"response": response}
            )
        return True

    async def enable_url(self, url: str) -> bool:
        """Enable a filter subscription in AdGuard Home."""
        response = await self._adguard._request(
            "filtering/enable_url", method="POST", data="url={}".format(url)
        )
        if response.rstrip() != "OK":
            raise AdGuardHomeError(
                "Failed enabling URL on AdGuard Home filter", {"response": response}
            )
        return True

    async def disable_url(self, url: str) -> bool:
        """Disable a filter subscription in AdGuard Home."""
        response = await self._adguard._request(
            "filtering/disable_url", method="POST", data="url={}".format(url)
        )
        if response.rstrip() != "OK":
            raise AdGuardHomeError(
                "Failed disabling URL on AdGuard Home filter", {"response": response}
            )
        return True

    async def refresh(self, force=False) -> bool:
        """Reload filtering subscriptions from URLs specified in AdGuard Home."""
        force = "true" if force else "false"
        response = await self._adguard._request(
            "filtering/refresh", method="POST", params={"force": force}
        )
        if not response.startswith("OK"):
            raise AdGuardHomeError(
                "Failed refreshing filter URLs in AdGuard Home", {"response": response}
            )
        return True



# -*- coding: utf-8 -*-
"""Asynchronous Python client for the AdGuard Home API."""

from .exceptions import AdGuardHomeError


class AdGuardHomeSafeSearch:
    """Controls AdGuard Home safe search enforcing."""

    def __init__(self, adguard):
        """Initialize object."""
        self._adguard = adguard

    async def enabled(self) -> bool:
        """Return if AdGuard Home safe search enforcing is enabled or not."""
        response = await self._adguard._request("safesearch/status")
        return response["enabled"]

    async def enable(self) -> bool:
        """Enable AdGuard Home safe search enforcing."""
        response = await self._adguard._request("safesearch/enable", method="POST")
        if response.rstrip() != "OK":
            raise AdGuardHomeError(
                "Enabling AdGuard Home safe search failed", {"response": response}
            )
        return True

    async def disable(self) -> bool:
        """Disable AdGuard Home safe search enforcing."""
        response = await self._adguard._request("safesearch/disable", method="POST")
        if response.rstrip() != "OK":
            raise AdGuardHomeError(
                "Disabling AdGuard Home safe search failed", {"response": response}
            )
        return True



# -*- coding: utf-8 -*-
"""Asynchronous Python client for the AdGuard Home API."""
import asyncio
import json
import socket

import aiohttp
import async_timeout
from yarl import URL

from .__version__ import __version__
from .exceptions import AdGuardHomeConnectionError, AdGuardHomeError
from .filtering import AdGuardHomeFiltering
from .parental import AdGuardHomeParental
from .querylog import AdGuardHomeQueryLog
from .safebrowsing import AdGuardHomeSafeBrowsing
from .safesearch import AdGuardHomeSafeSearch
from .stats import AdGuardHomeStats


class AdGuardHome:
    """Main class for handling connections with AdGuard Home."""

    def __init__(
        self,
        host: str,
        base_path: str = "/control",
        loop=None,
        password: str = None,
        port: int = 3000,
        request_timeout: int = 10,
        session=None,
        tls: bool = False,
        username: str = None,
        verify_ssl: bool = True,
        user_agent: str = None,
    ):
        """Initialize connection with AdGuard Home."""
        self._loop = loop
        self._session = session
        self._close_session = False

        self.base_path = base_path
        self.host = host
        self.password = password
        self.port = port
        self.request_timeout = request_timeout
        self.tls = tls
        self.username = username
        self.verify_ssl = verify_ssl
        self.user_agent = user_agent

        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._session is None:
            self._session = aiohttp.ClientSession(loop=self._loop)
            self._close_session = True

        if self.user_agent is None:
            self.user_agent = "PythonAdGuardHome/{}".format(__version__)

        if self.base_path[-1] != "/":
            self.base_path = self.base_path + "/"

        self.filtering = AdGuardHomeFiltering(self)
        self.parental = AdGuardHomeParental(self)
        self.querylog = AdGuardHomeQueryLog(self)
        self.safebrowsing = AdGuardHomeSafeBrowsing(self)
        self.safesearch = AdGuardHomeSafeSearch(self)
        self.stats = AdGuardHomeStats(self)

    async def _request(
        self, uri: str, method: str = "GET", data=None, json_data=None, params=None
    ):
        """Handle a request to the AdGuard Home instance."""
        scheme = "https" if self.tls else "http"
        url = URL.build(
            scheme=scheme, host=self.host, port=self.port, path=self.base_path
        ).join(URL(uri))

        auth = None
        if self.username and self.password:
            auth = aiohttp.BasicAuth(self.username, self.password)

        headers = {
            "User-Agent": self.user_agent,
            "Accept": "application/json, text/plain, */*",
        }

        try:
            with async_timeout.timeout(self.request_timeout):
                response = await self._session.request(
                    method,
                    url,
                    auth=auth,
                    data=data,
                    json=json_data,
                    params=params,
                    headers=headers,
                    ssl=self.verify_ssl,
                )
        except asyncio.TimeoutError as exception:
            raise AdGuardHomeConnectionError(
                "Timeout occurred while connecting to AdGuard Home instance."
            ) from exception
        except (aiohttp.ClientError, socket.gaierror) as exception:
            raise AdGuardHomeConnectionError(
                "Error occurred while communicating with AdGuard Home."
            ) from exception

        content_type = response.headers.get("Content-Type", "")
        if (response.status // 100) in [4, 5]:
            contents = await response.read()
            response.close()

            if content_type == "application/json":
                raise AdGuardHomeError(
                    response.status, json.loads(contents.decode("utf8"))
                )
            raise AdGuardHomeError(
                response.status, {"message": contents.decode("utf8")}
            )

        if "application/json" in response.headers["Content-Type"]:
            return await response.json()
        return await response.text()

    async def protection_enabled(self) -> bool:
        """Return if AdGuard Home protection is enabled or not."""
        response = await self._request("status")
        return response["protection_enabled"]

    async def enable_protection(self) -> bool:
        """Enable AdGuard Home protection."""
        response = await self._request("enable_protection", method="POST")
        if response.rstrip() != "OK":
            raise AdGuardHomeError(
                "Enabling AdGuard Home protection failed", {"response": response}
            )
        return True

    async def disable_protection(self) -> bool:
        """Disable AdGuard Home protection."""
        response = await self._request("disable_protection", method="POST")
        if response.rstrip() != "OK":
            raise AdGuardHomeError(
                "Disabling AdGuard Home protection failed", {"response": response}
            )
        return True

    async def version(self) -> str:
        """Return the current version of the AdGuard Home instance."""
        response = await self._request("status")
        return response["version"]

    async def close(self) -> None:
        """Close open client session."""
        if self._close_session:
            await self._session.close()

    async def __aenter__(self) -> "AdGuardHome":
        """Async enter."""
        return self

    async def __aexit__(self, *exc_info) -> None:
        """Async exit."""
        await self.close()



# -*- coding: utf-8 -*-
"""Asynchronous Python client for the AdGuard Home API."""

from .exceptions import AdGuardHomeError


class AdGuardHomeParental:
    """Controls AdGuard Home parental control."""

    def __init__(self, adguard):
        """Initialize object."""
        self._adguard = adguard

    async def enabled(self) -> bool:
        """Return if AdGuard Home parental control is enabled or not ."""
        response = await self._adguard._request("parental/status")
        return response["enabled"]

    async def enable(self) -> bool:
        """Enable AdGuard Home parental control."""
        response = await self._adguard._request(
            "parental/enable", method="POST", data="sensitivity=TEEN"
        )
        if response.rstrip() != "OK":
            raise AdGuardHomeError(
                "Enabling AdGuard Home parental control failed", {"response": response}
            )
        return True

    async def disable(self) -> bool:
        """Disable AdGuard Home parental control."""
        response = await self._adguard._request("parental/disable", method="POST")
        if response.rstrip() != "OK":
            raise AdGuardHomeError(
                "Disabling AdGuard Home parental control failed", {"response": response}
            )
        return True



# -*- coding: utf-8 -*-
"""Exceptions for AdGuard Home."""


class AdGuardHomeError(Exception):
    """Generic AdGuard Home exception."""

    pass


class AdGuardHomeConnectionError(AdGuardHomeError):
    """AdGuard Home connection exception."""

    pass



# -*- coding: utf-8 -*-
"""Asynchronous Python client for the AdGuard Home API."""

from .exceptions import AdGuardHomeError


class AdGuardHomeStats:
    """Provides stats of AdGuard Home."""

    def __init__(self, adguard):
        """Initialize object."""
        self._adguard = adguard

    async def dns_queries(self) -> int:
        """Return number of DNS queries."""
        response = await self._adguard._request("stats")
        return response["dns_queries"]

    async def blocked_filtering(self) -> int:
        """Return number of blocked DNS queries."""
        response = await self._adguard._request("stats")
        return response["blocked_filtering"]

    async def blocked_percentage(self) -> float:
        """Return the blocked percentage ratio of DNS queries."""
        response = await self._adguard._request("stats")
        return (response["blocked_filtering"] / response["dns_queries"]) * 100.0

    async def replaced_safebrowsing(self) -> int:
        """Return number of blocked pages by safe browsing."""
        response = await self._adguard._request("stats")
        return response["replaced_safebrowsing"]

    async def replaced_parental(self) -> int:
        """Return number of blocked pages by parental control."""
        response = await self._adguard._request("stats")
        return response["replaced_parental"]

    async def replaced_safesearch(self) -> int:
        """Return number of enforced safe searches."""
        response = await self._adguard._request("stats")
        return response["replaced_safesearch"]

    async def avg_processing_time(self) -> float:
        """Return avarage processing time of DNS queries (in ms)."""
        response = await self._adguard._request("stats")
        return response["avg_processing_time"]

    async def period(self) -> str:
        """Return the period the stats currently represent."""
        response = await self._adguard._request("stats")
        return response["stats_period"]

    async def reset(self) -> bool:
        """Reset all stats."""
        response = await self._adguard._request("stats_reset", method="POST")
        if response.rstrip() != "OK":
            raise AdGuardHomeError(
                "Resetting AdGuard Home stats failed", {"response": response}
            )
        return True
